//--------------------------------------------------------------------------------------
/**
 * <p>Title: CrashFaultInjector </p>
 * <b> Category: FT Baseline </b>
 * <b> Behavior: </b> Serving up a list of servers and letting you kill them at will
 * <b> Inherits from: </b>  java.lang.Thread
 * <b> Clients: </b> N/A
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//--------------------------------------------------------------------------------------
//plans for the future include:
//	Checking the JNDI to only show those servers that are active
//	Being able to kill at specific points of execution


package powerquotes.client;

//import java.util.logging.Level;

import java.io.*;
import java.util.*;
import powerquotes.ejb.*;
import powerquotes.common.*;
import org.apache.log4j.Logger;
import org.apache.log4j.Level;

public class CrashFaultInjector {

	private static String uid;
	private static Vector serverList;
	private static Properties props;
	

    //for logging
	protected static Logger log;

    /**
     * Default constructor.  Sets up logging
     */

    static {
            log = Logger.getLogger("powerquotes.client.CrashFaultInjector");
    }

   //================================================================================
	// Accessor Functions
    //================================================================================

	/**
	* Gets the machine name from calling function, 
	* ssh's with the default user's information
	* to the machine, and searches all the processes 
	* to kill all procs associated with the server on a given machine.
	* @param machineName name of the machine whose server is to be killed
	* @param uid ID of the default user... set on deployment
	* @returns true if everything went alright
	*/

	public static boolean killServer (String machineName, String uid){
		Runtime r=Runtime.getRuntime();
       	Process p =null;
       	String cmd="ssh " + uid +  "@" + machineName +
		" ps -eaf | grep java | grep jboss | grep team1 " +
        "| awk '{print $2}' | grep -v grep | xargs kill -9";
      	try{
			p=r.exec(cmd);
			if(p.waitFor() != 0){
				log.log(Level.ALL, ("killServer encountered a problem.\n" + 
							machineName + "may not have shut down..."));
                return false;
			}
		} catch(Exception e){
			log.log(Level.ALL, "killServer caught an exception...\n");
		}
		p.destroy();
        return true;
	}

	private static int mainMenu() {
        /* This is the main menu displayed to the user. */
        
		boolean err = true;
    	String userChoice = "";
		int checkChoice = -1;

        while (err) {
			System.out.println("\nChoose one of the servers below to kill\n");

			//assuming non-zero, unique server identifiers (1, 2, 3, etc)...
			for (int i = 0;i <= serverList.size(); ++i){
				System.out.println("Server "  + i + ": " +
				(String)serverList.elementAt(i));
			}
			System.out.print("\n\nEnter the number of your choice to kill a server," +
							"\n [X] to exit," +
							"\n Then press return >> ");
            
        	BufferedReader terminalReader = new BufferedReader(new InputStreamReader(System.in));
        	try {
            	userChoice = terminalReader.readLine();
	        } catch (IOException IOError) {
 	           log.log(Level.ALL, ("Error readking user input in CrashFaultInjector menu"));
  	      	}

        	if(userChoice.equals("X")){
				return -1;
       		} else{
        		try{
            		checkChoice = (new Integer(userChoice)).intValue();
         		} catch (NumberFormatException NFerror){
            		System.out.println("Improper input.  please try again.");
         		}
            	if (checkChoice != -1){
					if ((checkChoice >= 0) && (checkChoice < serverList.size())){
                		err = false;
            		} else {
                		System.out.println("Invalid input");
            		}
        		}
    		}
        }
        return checkChoice;
    }



    //================================================================================
	// Main
    //================================================================================
    /**
    * CrashFaultInjector main loads the properties file,
	* populates the serverList,
	* prompts for a server to kill...
	* main also keeps bringing up the main menu 
	* until the exit option i selected.
    */
    public static void main(String[] args){
        org.apache.log4j.xml.DOMConfigurator.configure("log4j.xml");

        //properties file is assumed to be 1 line per server,
        //of the form <machineName> = <serverNumberDesignation>
		props = new Properties();
        try{
 			props.load(new FileInputStream("deployment.properties"));
        } catch (IOException e) {
			log.log(Level.ALL, "I/O Error reading server deployment.properties files for killServer.");
			System.out.println("File not found.  terminating Injector.");
			System.exit(0);
        }

	    Enumeration keyEnum = props.propertyNames();
	    System.out.println("Enumerating");
		while (keyEnum.hasMoreElements())
		    {
			    String serverLoc = (String) keyEnum.nextElement();
			    int listIndex = (new Integer(props.getProperty(serverLoc))).intValue();
			    serverList.ensureCapacity(listIndex);
			    serverList.add(listIndex, serverLoc);
			    System.out.println("shibby");
		    }
//	    }
       uid = "ksteppe";

	    int serverIndex = -2;
	    while (serverIndex != -1)
	    {
		    serverIndex = mainMenu();
		    if (serverIndex != -1)
		    {
			    String machineName =
			            (String) serverList.elementAt(serverIndex);
			    killServer(machineName, uid);
		    }
		    else
		    {
			    System.exit(0);
		    }
	    }
    }
}
