//--------------------------------------------------------------------------------------
/**
 * <p>Title: StockListDisplay </p>
 * <b> Category: </b> Baseline
 * <b> Behavior: </b> Displays a list of stocks.
 * <b> Inherits from: </b> javax.swing.JPanel
 * <b> Clients: </b> N/A
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//--------------------------------------------------------------------------------------

package powerquotes.client;

//Import
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.util.*;
import powerquotes.common.*;

/**
 * Displays a list of stocks. 
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0 
 */
public class StockListDisplay extends JPanel
{
    // variables
    private JButton button;
    private JList list;

    // name of panel
    private String panelName;

    // stock list associated with panel
    Vector stocks = null;


    /**
     * Constructor with name and parent frame.
     */
    public StockListDisplay(String name, PowerQuotesFrame frame) {
        panelName = name;
        initComponents(frame);
    }


    /**
     * This method initializes the GUI components.
     * @param frame parent of the panel
     */
    private void initComponents(PowerQuotesFrame frame)
    {
        // set panel properties
        setLayout(new BorderLayout());

        // create components
        list = new JList();
        button = new JButton();

        // button text & listener
        button.setText(panelName);
        button.addActionListener(frame);

        list.setBorder(new LineBorder(new Color(0, 0, 0)));
        list.setModel(new AbstractListModel() 
        {
            String[] strings = { "", "" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });

        // add components
        add(list, BorderLayout.CENTER);
        add(button, BorderLayout.SOUTH);
    }


    //================================================================================
    // Accessor Functions
    //================================================================================
    /** Sets the stocks.
     * @param stocks list of stocks associated with the panel
     */
    public void setStocks(Vector stockList) 
    {
        stocks = stockList;

        // set the list box text
        Vector listData = new Vector();
        StockInfo stock;
        for(int i=0; i< stockList.size(); i++)
        {       
            stock = (StockInfo) stockList.elementAt(i);
            listData.add(stock.getSymbol());
        }

        list.setListData(listData);
    }

    /** Gets the stocks.
     * @return the list of stocks associated with the panel
     */
    public Vector getStocks() { return(this.stocks); }

    /**
     * Sets the panel name
     * @param name name of the panel
     */
    public void setPanelName(String name)
    {
        this.panelName = name;
        this.button.setText(name);
    }


    /**
     * Gets the panel name.
     * @return the name of the file
     */
    public String getPanelName() { return(this.panelName); }


    /**
     * Returns the stock ids.
     * @return the list of stock ids that are currently selected in the list box
     */
    public Vector getSelectedStocks()
    {
        // get the indices selected in the list box
        int[] indices = this.list.getSelectedIndices();

        // get the stock ids associated with these indices
        Vector selected = new Vector();

        for(int i=0; i<indices.length; i++)
        {
            selected.add(stocks.elementAt(indices[i]));
        }

        return selected;
    }
}
