/**
 * <p>Title: StockInfo </p>
 * <b> Category: </b> Baseline
 * <b> Behavior: </b> Provide information about a stock.
 * <b> Inherits from: </b> none
 * <b> Clients: </b> Shared
 * 
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 * 
 */
package powerquotes.common;


/**
 * Provides information about a stock.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0 
 */ 
public class StockInfo implements java.io.Serializable {
        
    private int id;
    private String symbol;
    private String name;
    private double price;
    private long timestamp;

    /**
     * Constructor with id, symbol, name, price, and timestamp.
     *
     * @param id unique ID for this stock
     * @param symbol the stock's ticker symbol
     * @param name the company's full name
     * @param price the current price of the stock
     * @param timestamp the last time the stock price has been updated,
     *                  in milliseconds since the Epoch. (i.e. Unix time)
     */
    public StockInfo( int id, String symbol, String name, double price, long timestamp ) {
        this.id = id;
        this.symbol = symbol;
        this.name = name;
        this.price = price;
        this.timestamp = timestamp;
    }

    /**
     *@return id of stock
     */
    public int getId() {
        return id;
    }

    /**
     *@return symbol associated with the stock
     */
    public String getSymbol() {
        return symbol;
    }

    /**
     *@return name of stock
     */
    public String getName() {
        return name;
    }

    /**
     *@return current price of stock
     */
    public double getPrice() {
        return price;
    }

    /**
     *@return timestamp associated with the current price
     */
    public long getTimestamp() {
        return timestamp;
    }

    /**
     *@param newPrice new stock price 
     */
    public void setPrice( double newPrice ) {
        price = newPrice;
    }

    /**
     *@param newTimestamp timestampe associated with new price
     */
    public void setTimestamp( long newTimestamp ) {
        timestamp = newTimestamp;
    }
    
    /**
     *@return string representation of stock
     */
    public String toString() {
        return new String( "symbol: " + symbol + ", id: " + id + ", name: " + name + ", price: " + price );
    }
}
