//------------------------------------------------------------------------------------
/**
 * <p>Title: UserProfile </p>
 * <b> Category: </b> Baseline
 * <b> Behavior: </b> User profile for Stock Quote application
 * <b> Inherits from: </b> none
 * <b> Clients: </b> ClientApplication
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//------------------------------------------------------------------------------------
package powerquotes.common;

// Import
import java.io.Serializable;
import java.util.*;


/**
 * User profile for Stock Quote application.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0 
 */
public class UserProfile implements Serializable {

	// variables
	private String userName;
	private int userId;
	private String profileName;
	private int profileId;
	private int stockList[] = {};


 	/** Default Constructor. */
 	public UserProfile() {}

    
	/** Constructor with user name and stock list.
	 *
	 * @param name user name associated with profile
	 * @param list list of stocks the user is interested in seeing updates for
	 */
	public UserProfile(String name, int[] list) {
		this.userName = name;
		this.stockList = list;
	}


	/**
	 * Adds stocks to the stockList.
	 */
	public void addStocks(Vector add)
	{
		// **Note: this is a really bad way to do this.
		Vector newStocks = new Vector();

		// put current stocks in here
		for(int i=0; i<stockList.length; i++)
		{
			newStocks.add(new Integer(stockList[i]));
		}

		// add new ones that are not duplicates
		StockInfo info;
		for(int i=0; i<add.size(); i++)
		{
			info = (StockInfo) add.elementAt(i);
			if(!newStocks.contains(new Integer(info.getId())))
			{
				newStocks.add(new Integer(info.getId()));
			}
		}

		// put into int array
		int[] newArray = new int[newStocks.size()];
		Integer integer;
		for(int i=0; i< newArray.length; i++)
		{
			integer = (Integer) newStocks.elementAt(i);
			newArray[i] = integer.intValue();
		}

		// set stock List
		stockList = newArray;
	}


	/**
	 * Removes stocks from the stockList
	 */
	public void removeStocks(Vector remove)
	{
		//**Note:  this is a really bad way to do this.
		Vector newStocks = new Vector();

		// put current stocks in here
		for(int i=0; i<stockList.length; i++)
		{
			newStocks.add(new Integer(stockList[i]));
		}

		// get rid of the ones that are in remove
		StockInfo info;
		for(int i=0; i<remove.size(); i++)
		{
			info = (StockInfo) remove.elementAt(i);
			if(newStocks.contains(new Integer(info.getId())))
			{
				newStocks.remove(new Integer(info.getId()));
			}
		}

		// put into int array
		int[] newArray = new int[newStocks.size()];
		Integer integer;
		for(int i=0; i< newArray.length; i++)
		{
			integer = (Integer) newStocks.elementAt(i);
			newArray[i] = integer.intValue();
		}

		// set stock List
		stockList = newArray;
	}


    //================================================================================
	// Accessor Functions
    //================================================================================
    /** Sets the userName.
     * @param name user name associated with profile */
	public void setUserName(String name) { this.userName = name; }

	/** Gets the userName.
	* @return returns the user name associated with profile */
	public String getUserName() { return(this.userName); }

	/** Sets the userId.
	 * @param id ID number associated with profile */
	public void setUserId(int id) { this.userId = id; }

 	/** Gets the userId.
 	* @return returns the ID number associated with profile */
	public int getUserId() { return(this.userId); }

	/** Sets the profileName */
	public void setProfileName(String name) { this.profileName = name; }

	/** Gets the profileName */
	public String getProfileName() { return(this.profileName); }

	/** Sets the profileId. */
	public void setProfileId(int id) { this.profileId = id; }

 	/** Gets the profileId. */
	public int getProfileId() { return(this.profileId); }

	/** Sets the stockList array.
	* @param list list of stocks the user is interested in seeing updates for */
	public void setStockList(int[] list) { this.stockList = list; }

	/** Gets the stockList array.
	* @return returns the list of stocks the user is interested in seeing updates for */
	public int[] getStockList() { return(this.stockList); }

}
