//--------------------------------------------------------------------------------------
/**
 * <p>Title: Utilities </p>
 * <b> Category: </b> Baseline
 * <b> Behavior: </b> Provides utility functions to the server & client.
 * <b> Inherits from: </b> none
 * <b> Clients: </b> N/A
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//--------------------------------------------------------------------------------------
package powerquotes.common;

// Import
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.*;
import java.rmi.RemoteException;
import powerquotes.ejb.*;
import org.apache.log4j.Logger;

import javax.naming.NamingException;
import java.lang.ClassCastException;
import javax.ejb.CreateException;
import java.rmi.RemoteException;

/*
 * Provides utility functions to the server & client.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
public class Utilities
{

    protected static Logger log = Logger.getLogger("powerquotes.common.Utilities");

    /**
     * This method binds to the UserProfileSessionBean
     *
     * @param num num to append to bean name, -1 if no appending desired
     * @return UPSession bean object
     * @throws StockServerException if something went wrong
     */
    public static UPSession bindToUserProfileSession(int num)
        throws StockServerException
    {

        UPSession upSession = null;
        String homeName = "ejb/powerquotes/UPSession";

        // append the _num to the homeName
        if(num != -1)
            homeName = homeName.concat("_" + num);

        // Obtain initial context
        InitialContext initialContext = null;

        try
        {
            initialContext = new InitialContext();
            Object objRef = initialContext.lookup(homeName);
            UPSessionHome home = (UPSessionHome) PortableRemoteObject.narrow(objRef, UPSessionHome .class);
            upSession = home.create();
        }
        catch(NamingException ne)
        {
            // NamingException if naming exception encountered with the Context
            log.error("Caught a naming exception! " + ne.toString(), ne);
            throw new StockServerException();
        }
        catch(ClassCastException cce)
        {
            // ClassCastException if PortableRemoteObject cannot cast to UPSessionHome interface
            log.error("Caught a class cast exception! " + cce.toString(), cce);
            throw new StockServerException();
        }
        catch(CreateException ce)
        {
            // CreateException if exception encountered whil creating the bean
            log.error("Caught a create exception! " + ce.toString(), ce);
            throw new StockServerException();
        }
        catch(RemoteException re)
        {
            // RemoteException if exception encountered while connecting to server
            log.error("Caught a remote exception! " + re.toString(), re);
            throw new StockServerException();
        }
        return upSession;
    }


    /**
     * This method binds to the ClientUpdateSessionBean
     *
     * @param num num to append to bean name, -1 if no appending desired
     * @return ClientUpdate bean object
     * @throws StockServerException if something went wrong
     */
    public static ClientUpdate bindToClientUpdateSession(int num)
        throws StockServerException
    {
        ClientUpdate cUpdate = null;
        String homeName = "ejb/powerquotes/ClientUpdate";

        // append the _num to the homeName
        if(num != -1)
            homeName = homeName.concat("_" + num);

        // Obtain initial context
        InitialContext initialContext = null;

        try {
            initialContext = new InitialContext();
            Object objRef = initialContext.lookup(homeName);
            ClientUpdateHome updateHome = (ClientUpdateHome) PortableRemoteObject.narrow(objRef, ClientUpdateHome .class);
            cUpdate = updateHome.create();
        }
        catch(NamingException ne)
        {
            // NamingException if naming exception encountered with the Context
            log.error("Caught a naming exception! " + ne.toString(), ne);
            throw new StockServerException();
        }
        catch(ClassCastException cce)
        {
            // ClassCastException if PortableRemoteObject cannot cast to UPSessionHome interface
            log.error("Caught a class cast exception! " + cce.toString(), cce);
            throw new StockServerException();
        }
        catch(CreateException ce)
        {
            // CreateException if exception encountered whil creating the bean
            log.error("Caught a create exception! " + ce.toString(), ce);
            throw new StockServerException();
        }
        catch(RemoteException re)
        {
            // RemoteException if exception encountered while connecting to server
            log.error("Caught a remote exception! " + re.toString(), re);
            throw new StockServerException();
        }

        return cUpdate;
    }

    /**
     * This method binds to the StateReplicatorSessionBean
     *
     * @param num num to append to bean name, -1 if no appending desired
     * @return  StateReplicatorSessionBean object
     * @throws StockServerException if something went wrong
     */
    public static StateReplicator bindToStateReplicator(int num)
	    throws StockServerException

    {
        StateReplicator replicator = null;
        String homeName = "ejb/powerquotes/StateReplicator";

        // append the _num to the homeName
        if(num != -1)
            homeName = homeName.concat("_" + num);

        // Obtain initial context
        log.debug( "bindToStateReplicator - homeName: " + homeName );

        try {
            InitialContext initialContext = new InitialContext();
            Object objRef = initialContext.lookup(homeName);
            StateReplicatorHome replicatorHome =
                (StateReplicatorHome) PortableRemoteObject.narrow(objRef, StateReplicatorHome .class);
            replicator = replicatorHome.create();
        }
        catch(NamingException ne)
        {
            // NamingException if naming exception encountered with the Context
            log.error("Caught a naming exception! " + ne.toString(), ne);
            throw new StockServerException();
        }
        catch(ClassCastException cce)
        {
            // ClassCastException if PortableRemoteObject cannot cast to UPSessionHome interface
            log.error("Caught a class cast exception! " + cce.toString(), cce);
            throw new StockServerException();
        }
        catch(CreateException ce)
        {
            // CreateException if exception encountered whil creating the bean
            log.error("Caught a create exception! " + ce.toString(), ce);
            throw new StockServerException();
        }
        catch(RemoteException re)
        {
            // RemoteException if exception encountered while connecting to server
            log.error("Caught a remote exception! " + re.toString(), re);
            throw new StockServerException();
        }

        return replicator;
    }
}
