/**
 * <p>Title: SequenceGenHelper</p>
 * <b> Category:  </b> Baseline
 * <b> Behavior:</b> Provides help to the sequence generator.
 * <b> Inherits from:</b> none
 * <b> Clients:</b> N/A
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
package powerquotes.ejb.impl;

// Import
import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;

import javax.naming.Context;
import javax.naming.InitialContext;

import powerquotes.ejb.*;
import org.apache.log4j.Logger;


/*
 * Provides help to the sequence generator.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0 
 */
public class SequenceGenHelper {

    protected static Logger log = Logger.getLogger("powerquotes.ejb.impl.SequenceGenHelper");

    private static SequenceGenHelper theInstance = null;
    private SequenceGeneratorLocal lBean = null;
    private String lSequenceName = null;

    private SequenceGenHelper() throws ServiceUnavailableException {

         try {
            SequenceGeneratorLocalHome sglh = SequenceGeneratorUtil.getLocalHome();
            lBean = (SequenceGeneratorLocal) sglh.create();

            Context lContext = new InitialContext();
            lSequenceName = (String) lContext.lookup(
                "java:comp/env/SequenceName"
                );

        } catch ( NamingException ne ) {
            throw new ServiceUnavailableException( "Naming lookup failure: " + ne.getMessage() );
        }
        catch ( CreateException ce ) {
            throw new ServiceUnavailableException( "Failure while creating a generator session bean: " + ce.getMessage() );
        }

    }

    /**
     * @return the instance of the SequenceGenHelper
     * @throws ServiceUnavailableException Indicating that it was not possible
     *                                     to retrieve a new unqiue ID because
     *                                     the service is not available
     */
    public static SequenceGenHelper getInstance() throws ServiceUnavailableException {

        if( theInstance == null ) {
            theInstance = new SequenceGenHelper();
        }
        return theInstance;
    }


   /**
   * Retrive a unique creation id to use for this bean.  This will end up
   * demarcating this bean from others when it is stored as a record
   * in the database.
   *
   * @return Returns an integer that can be used as a unique creation id.
   *
   * @throws ServiceUnavailableException Indicating that it was not possible
   *                                     to retrieve a new unqiue ID because
   *                                     the service is not available
   **/
   public int generateUniqueId( String pkFieldName )
      throws ServiceUnavailableException
   {
      int lUniqueId = -1;
      try {
          lUniqueId = lBean.getNextNumber( lSequenceName, pkFieldName );
          lBean.remove();
      } catch ( RemoveException re ) {
            // When the Bean cannot be removed after a while it will be taken back by the container
            // therefore ignore this exception
      }
      return lUniqueId;
   }

}
