//------------------------------------------------------------------------------------
/**
 * <p>Title: TickReceiverSessionBean </p>
 * <b> Category:  </b> Baseline
 * <b> Behavior:</b> Provides a way to transfer state between servers.
 * <b> Inherits from:</b> javax.ejb.SessionBean
 * <b> Clients:</b> PowerQuotesClient
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//------------------------------------------------------------------------------------

package powerquotes.ejb.impl;

// Import
import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import powerquotes.ejb.*;
import powerquotes.common.*;

import java.util.HashMap;
import org.apache.log4j.Logger;


/**
 * Provides a way to transfer state between servers.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 *
 * @ejb:bean name="StateReplicator"
 *           display-name="State Replicator Session Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/powerquotes/StateReplicator"
 *           view-type="both"
 * @ejb:home remote-package="powerquotes.ejb"
 * @ejb:interface remote-package="powerquotes.ejb"
 * @ejb:util generate="physical"
*/
public class StateReplicatorSessionBean implements SessionBean {

    protected static Logger log = Logger.getLogger("powerquotes.ejb.impl.StateReplicatorSessionBean");


    //================================================================================
    // Client Interface Methods
    //================================================================================
    /** This method retrieves the stock price cache from the server and returns it.
     * This will be used to transfer the cache state to other servers.  It is also
     * used by the test clients to determine current state.
     *
     * @category Fault Tolerance
     * @clients Servers and Tests
     * @return StockCache
     *
     * @ejb:interface-method view-type="remote"
     */
    public StockCache getStockCache()
    {
        StockCache cache = StockCache.getInstance();
        System.out.println("Pulling out cache: " + cache);
        return cache;
    }

    /**
     *
     * @category Fault Tolerance
     * @clients Replication managers
     * @return number of clients subscribed on the server
     *
     * @ejb:interface-method view-type="remote"
     */
    public int ping()
    {
        return Subscription.getInstance().getNumSubscriptions();
    }

   /**
     *
     * @category Fault Tolerance
     * @clients Replication managers
     *
     * @ejb:interface-method view-type="remote"
     */
    public void initializeCache( StockCache otherCache )
    {
	    log.debug("init cache");
	    StockCache cache = StockCache.getInstance();
	    log.debug("cache: " + cache.toString());
	    cache.initializeCache( otherCache );
	    log.debug("cache ready");

    }

    //================================================================================
    // EJB Methods
    //================================================================================

    /** Create the Session Bean.
     *
     * @throws CreateException
     *
     * @ejb:create-method view-type="remote"
     */
    public void ejbCreate() throws CreateException {
        log.debug("StateReplicatorSessionBean: ejbCreate()");
    }
    public void setSessionContext(SessionContext sc) {
        log.debug("StateReplicatorSessionBean: setSessionContext()");
    }
    public void ejbRemove() {
        log.debug("StateReplicatorSessionBean: ejbRemove()");
    }
    public void ejbActivate() {
        log.debug("StateReplicatorSessionBean: ejbActivate()");
    }
    public void ejbPassivate() {
        log.debug("StateReplicatorSessionBean: ejbPassivate()");
    }

}
