/**
 * <p>Title: Subscription </p>
 * <b> Category:</b>  Baseline
 * <b> Behavior:</b>  Keeps track of client subscriptions.
 * <b> Inherits from:</b>  N/A
 * <b> Clients:</b> N/A
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
package powerquotes.ejb.impl;


// Import
import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import java.rmi.RemoteException;
import javax.naming.Context;
import javax.naming.InitialContext;
import java.util.*;

import powerquotes.ejb.*;
import org.apache.log4j.Logger;


/**
 * Singleton class that represents the system's current knowledge about client subscriptions
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0 
 */
public class Subscription {

	// variables
	private static Subscription theSubs = null;
	private HashMap subs = null;

    // we should A) get this from a config file, B) use a real logger
    private static final boolean debug = true;
    protected static Logger log = Logger.getLogger("powerquotes.ejb.impl.Subscription");


	/**
	 * @return the instance of the Subscription
	 */
    public static Subscription getInstance()
    {
        if(theSubs == null)
        {
            theSubs = new Subscription ();
        }
        return theSubs;
    }


	/**
	 * This method creates the instance of Subscription.
	 */
    private Subscription ()
    {
		subs = new HashMap();
    }


    /**
     * This method sets the subscription information for a user.
     *
     * @param userName name of user associated with subscription
     * @param list of stocks user is subscribing to
     */
    public void setSubscription(String userName, int[] stockId)
    {
		int[] oldInfo = (int []) subs.put(userName, stockId);

        log.debug("Updated Cache with: " + stockId);
        log.debug("Old info was: " + oldInfo);
    }


    /**
     * This method gets the subscription information for a user.
     *
     * @return the stock ids for the user or null if it wasn't found
     */
    public int[] getSubscription(String userName)
    {
		return (int[]) subs.get(userName);
	}

    public int getNumSubscriptions()
    {
        if (subs != null)
            return subs.size();
        else
            return 0;
    }
    
}


