//------------------------------------------------------------------------------------
/**
 * <p>Title: TickReceiverSessionBean </p>
 * <b> Category: </b> Baseline
 * <b> Behavior:</b>  Receives stock updates from the stock ticker.
 * <b> Inherits from:</b> javax.ejb.SessionBean
 * <b> Clients:</b> PowerQuotesClient
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//------------------------------------------------------------------------------------

package powerquotes.ejb.impl;

// Import
import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import powerquotes.ejb.*;
import powerquotes.common.*;

import java.util.HashMap;
import org.apache.log4j.Logger;


/** 
 * Receives stock updates from the stock ticker.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 * 
 * @ejb:bean name="TickReceiver"
 *           display-name="Tick Receiver Session Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/powerquotes/TickReceiver"
 *           view-type="remote"
 * @ejb:home remote-package="powerquotes.ejb"
 * @ejb:interface remote-package="powerquotes.ejb"
 * @ejb:util generate="physical"
*/
public class TickReceiverSessionBean implements SessionBean {


    protected static Logger log = Logger.getLogger("powerquotes.ejb.impl.TickReceiverSessionBean");


    //================================================================================
    // Client Interface Methods
    //================================================================================

    /** This method accepts a stock tick.  In the baseline it will put the update into
     * the cache.  It would also store to the database if we were doing that.
     * In the pub-sub version this call the subscription cache to  publish stock updates
     * to attached clients.
     *
     * FT: must not lose any data updates.
     * Performance: update must reach all clients within 5 seconds of generation.
     *
     * @category Baseline
     * @clients PowerQuotesClient
     * @param newInfo The new Stock information.
     * @return none
     *
     * @ejb:interface-method view-type="remote"
     */
    public void updateStockInfo(StockInfo newInfo)
    {
        //if we were storing history
            //if master?
                //store to db
                //tell backup I've saved to db
            //else skip

        //here I update my cache
        //StockCache cache = ???

        //first find the cache object
        StockCache cache = StockCache.getInstance();

        //update the cache's view of the object
        cache.setStock(newInfo);

        //call the subscription cache to publish update (in pub/sub mode only)
    }



    //================================================================================
    // EJB Methods
    //================================================================================

    /** Create the Session Bean.
     *
     * @throws CreateException
     *
     * @ejb:create-method view-type="remote"
     */
    public void ejbCreate() throws CreateException {
        log.debug("TickReceiverSessionBean: ejbCreate()");
    }
    public void setSessionContext(SessionContext sc) {
        log.debug("TickReceiverSessionBean: setSessionContext()");
    }
    public void ejbRemove() {
        log.debug("TickReceiverSessionBean: ejbRemove()");
    }
    public void ejbActivate() {
        log.debug("TickReceiverSessionBean: ejbActivate()");
    }
    public void ejbPassivate() {
        log.debug("TickReceiverSessionBean: ejbPassivate()");
    }

}
