/**
 * <p>Title: UserProfileBean </p>
 * <b> Category: </b> Baseline
 * <b> Behavior: </b> User profile EntityBean for Stock Quote application.
 * <b> Inherits from:</b> javax.ejb.EntityBean
 * <b> Clients: </b> ClientApplication
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
package powerquotes.ejb.impl;

// Import
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;

/**
 * User profile EntityBean for Stock Quote application.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 *
 * XDoclet Tags:
 *
 * ATTENTION: Some of the XDoclet tags are hidden from XDoclet by
 *            adding a "--" between @ and the namespace. Please remove
 *            this "--" to make it active or add a space to make an
 *            active tag inactive.
 *
 * @ejb:bean name="UserProfileBean"
 *           display-name="UserProfile EntityBean"
 *           type="CMP"
 *           primkey-field="profileId"
 *           jndi-name="ejb/powerquotes/UserProfileRemote"
 *           local-jndi-name="ejb/powerquotes/UserProfileLocal"
 *           view-type="both"
 *
 * @ejb:home remote-package="powerquotes.ejb"
 *           local-package="powerquotes.ejb"
 *
 * @ejb:interface remote-package="powerquotes.ejb"
 *                local-package="powerquotes.ejb"
 *
 * @ejb:util generate="physical"
 *
 * @ejb:env-entry name="SequenceName"
 *                value="UserProfileBean"
 *
 * --@ejb:persistence table-name="Profiles"
 *
 * @ejb:pk  class="java.lang.Integer"
 *
 * @ejb:finder signature="UseProfileBeanLocal findByUserName( java.lang.String userName )"
 *                         query="SELECT DISTINCT OBJECT( o ) FROM UserProfileBean AS o WHERE o.userName=?1"
 *                         result-type-mapping="Local"
 *                         view-type="local"
 *
 *
 * @jboss.persistence create-table="true"
 *
 *
 */
public abstract class UserProfileBean implements EntityBean {


    /**
      * Retrieve the UserProfile's id. This is the PK for the bean.
      *
      * @return Returns an Integer representing the id of this user.
      *
      * @ejb:persistence column-name name="profileId"
      * @ejb:interface-method
      * @jboss:persistence not-null="true"
      *
      */
    public abstract Integer getProfileId();


      /**
      * Retrieve the userid of the user that owns this profile
      *
      * @return Returns an int representing the id of this user.
      *
      * @ejb:persistence column-name name="userId"
      * @ejb:interface-method
      *
      */
    public abstract int getUserId();

     /**
      * Retrieve the name for this profile
      *
      * @return Returns a string for the name of this profile
      *
      * @ejb:persistence column-name name="profileName"
      * @ejb:interface-method
      */
    public abstract String  getProfileName();

     /**
      * Retrieve the name for this profile
      *
      * @return Returns the name of the user associated to this profile
      *
      * @ejb:persistent-field column-name name="userName"
      * @ejb:interface-method
      */
    public abstract String  getUserName();



    /**
     * HELP! How do we do arrays??
      * Retrieve the name for this profile

      * @return Returns a string for the name of this profile
      *
      * @ejb:persistent-field column-name name="stockList"
      * @ejb:interface-method
      */
    public abstract int[] getStockList();



    /**
      * Setter for the profile id. This is set at creation time.
      * @param profileId the id for the profile
      *
      * @ejb:persistence column-name name="profileId"
      * @ejb:interface-method
      */
    public abstract void setProfileId( Integer userId );

    /**
      * Setter for the user id that owns this profile
      * @param userId user id to assign to this profile
      *
      * @ejb:persistence column-name name="profileName"
      * @ejb:interface-method
      */
    public abstract void setUserId( int userId );

     /**
      * Retrieve the name for this profile
      *
      * @return Sets name of the user to be associated to this profile
      * @ejb:persistence column-name name="userName"
      * @ejb:interface-method
      */
    public abstract void setUserName( String userName );

    /**
      * Setter for the name for this profile
      * @param userId new name for the profile
      *
      * @ejb:persistence column-name name="profileName"
      * @ejb:interface-method
      *
      */

    public abstract void setProfileName( String proFileName );

    /**
      * Setter for the stock list
      * @param stockList list of stocks for  this profile
      *
      * @ejb:persistent-field column-name name="stockList"
      * @ejb:interface-method
      */
    public abstract void setStockList( int stockList[] );



    /********************* Framework methods **********************/

    /**
     * Create a TestEntity based on the supplied TestEntity Value Object.
     *
     * @throws CreateException Because we have to do so (EJB spec.)
     *
     * @ejb:create-method
     **/
    public Integer ejbCreate( Integer id ) throws CreateException {
        try {
            SequenceGenHelper sgh = SequenceGenHelper.getInstance();
            id = new Integer( sgh.generateUniqueId( "profileId" ) );
        } catch( ServiceUnavailableException sue ) {
            throw new CreateException( "UserProfileBean could not be created becuase a new profile ID could not be generated.\n" + sue );
        }
	setProfileId( id );

        return null;
    }

    public void ejbPostCreate( Integer id ) {
    }

    public void setEntityContext( EntityContext ec ) {
    }

    public void unsetEntityContext() {
    }

    public void ejbActivate() {
    }

    public void ejbPassivate() {
    }

    public void ejbLoad() {
    }

    public void ejbStore() {
    }

    public void ejbRemove() throws RemoveException {
    }
/*****************  End Framework methods  *****************************/

/****************  Private methods   **********************************/




}


