package powerquotes.repmanager;

import powerquotes.common.StockCache;
import powerquotes.common.StockInfo;
import powerquotes.common.ServerNotReadyException;

//import powerquotes.common.Utilities;
import org.apache.log4j.Logger;
import powerquotes.ejb.StateReplicator;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

/**
 * Created by IntelliJ IDEA.
 * User: ksteppe
 * Date: Mar 9, 2003
 * Time: 3:23:06 PM
 * To change this template use Options | File Templates.
 */
public class GlobalReplicationManager implements Runnable
{
    protected static Logger log;
    protected int sleepPeriod = 60;
    protected int numServers = 0;
    protected ArrayList managers;
    protected boolean stopping = false;

    public GlobalReplicationManager(int period)
    {

        if (log == null)
            log = Logger.getLogger(this.getClass().getName());

        this.sleepPeriod = period;
    }

    /**
     *
     * @param args - number of servers to watch
     */
    public static void main (String[] args)
    {
        org.apache.log4j.xml.DOMConfigurator.configure("log4j.xml");

        if (args.length < 1)
            {
                usage();
                return;
            }

        int checkPeriod = Integer.parseInt(args[0]);
        GlobalReplicationManager grm = new GlobalReplicationManager(checkPeriod);
        //        grm.setCheckPeriod(checkPeriod);
        grm.run();
    }

    protected static void usage()
    {
        System.out.println("Usage: GlobalReplicationManager checkPeriod");
        System.out.println("checkPeriod -- number of seconds to sleep between checking on the servers");

    }

    public void run()
    {
        //distribute the server assignments
        log.debug("setting up managers");

        setup();

        //        bindServers();
        while (!stopping)
            {

                log.debug("in menu loop");

                //would like to have a menu here
                System.out.println("Options:");
                System.out.println("1) Halt Replication Managers (includes global)");
                System.out.println("2) Shutdown all servers (and replication managers)");
                System.out.println("3) Shutdown everything (excludes JNDI)");
	        System.out.println("4) Shutdown one server:");

                try
                    {
                        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
                        String choice = in.readLine();

                        if (choice.length() > 0)
                            {
                                switch (choice.charAt(0))
                                    {
                                    case '1':
                                        stopManagers(false);
                                        break;
                                    case '2':
                                        stopManagers(true); // = t
                                        break;
                                    case '3':
                                        System.out.println("not yet implemented.  sorry");
                                        break;
                                    case '4':
                                        for (int i = 0; i < managers.size(); i++)
                                            {
                                                ReplicationManager rm = (ReplicationManager)managers.get(i);
                                                System.out.println((i+1) + ") " + rm.getServerName());
                                            }
                                        choice = in.readLine();
                                        int c = Integer.parseInt(choice);
                                        ReplicationManager rm = (ReplicationManager)managers.get(c - 1);
                                        rm.haltServer();
                                    default:
                                    }
                            }
                    }
                catch (IOException ioe)
                    {
                        log.warn(ioe.toString(), ioe);
                    }

                //            try {
                //                Thread.sleep(1000 * this.getCheckPeriod());
                Thread.yield();
                //            }
                //            catch (InterruptedException ie)
                //            {
                //                swallow the interruption
                //            }
            }
    }

    protected void setup()
    {
        //get the number and names of servers?
        Properties props = new Properties();
        try{
            props.load(new FileInputStream("serverlist.props"));
        } catch (IOException ioe) {
            log.error("Error loading properties: " + ioe, ioe);
        }

        log.debug("Reading properties");

        managers = new ArrayList();
        int i = 0;
        Enumeration keyEnum = props.propertyNames();
        while (keyEnum.hasMoreElements())
            {
                String serverLoc = (String) keyEnum.nextElement();
                int listIndex = (new Integer(props.getProperty(serverLoc))).intValue();
                //			serverList.add(listIndex, serverLoc);

                //Server Id's start at 1 -- so I send i+1 to the constructor
                ReplicationManager rm = new ReplicationManager(listIndex, serverLoc,
                                                               this.getCheckPeriod(), this);
                rm.setCheckPeriod(this.getCheckPeriod());
                rm.setUID("ksteppe"); //<--- NOOOOO ks42

                //remember the indexes might get out of order somehow
                managers.add(rm);
                log.debug("starting manager: " + listIndex + " on: " + serverLoc);
                rm.start();
                //			Thread t = new Thread(rm);
                //		    t.start();
                i++;
            }
        this.setNumServers(managers.size());
        log.debug("Launched all servers");

    }

    protected void stopManagers(boolean halting)
    {

        for (int i = 0; i < managers.size(); i ++)
            {
                ReplicationManager rm = (ReplicationManager)(managers.get(i));
                //	        if (rm != null)
                //	        {
                if (halting)
                    {
                        log.info("Halting rep manager " + i);
                        rm.halt();
                    }
                else
                    {
                        log.info("Stopping rep manager " + i);
                        rm.stopManager();
                    }
                rm.interrupt();
                //	        }
            }
        stopping = true;

    }

    /**
     * Can't yet change the number of servers after start up.
     * @param num
     */
    private void setNumServers(int num)
    {
        this.numServers = num;
    }

    public int getNumServers()
    {
        return this.numServers;
    }

    public int getCheckPeriod()
    {
        return this.sleepPeriod;
    }

    public void setCheckPeriod(int secs)
    {
        this.sleepPeriod = secs;
        for (int i = 0; i < managers.size(); i++)
            {
                ReplicationManager rm = (ReplicationManager)(managers.get(i));
                rm.setCheckPeriod(secs);
            }
    }

    /**
     * @return the StockCache of another running server or null if none are alive
     * @param serverId the serverId of the server requesting the cache
     */
    public StockCache getOtherStockCache( int serverId )
    {
        Iterator it = managers.iterator();
        ReplicationManager manager = null;
        while( it.hasNext() ) {
            manager = (ReplicationManager) it.next();
            if( manager.isServerAlive() ) {
                StateReplicator replicator = manager.getReplicator();
                try {
                    StockCache cache = replicator.getStockCache();
	                log.debug("Got cache from: " + manager.getServerName());
	                outputCache(cache);
//                    log.debug( "cache: " + cache );
                    return cache;
                } catch( RemoteException re ) {
                    log.error( "Could not get stock cache from server " + manager.getServerId() +
                               "(" + manager.getServerName() + ")" );
                }
            }
        }
        return null;
    }

   public static void outputCache(StockCache cache)
    {
      //  try
       // {
//            System.out.println("have stateBean: " + stateBean);
//            cache = stateBean.getStockCache();
            log.debug("got cache: " + cache);

            if( cache != null ) {
                Collection stocks = cache.getInternalCache().values();
                for (Iterator i = stocks.iterator(); i.hasNext(); )
                    {
                        StockInfo si = (StockInfo)i.next();
                        log.debug(si.toString());
                    }
            }
       // }
       // catch(ServerNotReadyException se)
        //{
//            log.error("Caught a server not ready exception during cacheOutputTest!" + se.toString(), se);
        //}
//        catch (RemoteException re)
//        {
//            log.error(re.toString(), re);
//        }
    }
}
