package powerquotes.repmanager;

import powerquotes.common.StockCache;
import powerquotes.client.CrashFaultInjector;
import powerquotes.ejb.StateReplicator;
import powerquotes.ejb.StateReplicatorHome;
import org.apache.log4j.Logger;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.ejb.CreateException;
import java.io.IOException;
import java.rmi.RemoteException;

/**
 * Created by IntelliJ IDEA.
 * User: ksteppe
 * Date: Mar 9, 2003
 * Time: 4:19:31 PM
 * To change this template use Options | File Templates.
 */
public class ReplicationManager extends Thread// Runnable
{
    StateReplicator stateReplicator = null;

    protected int serverWatchedId;
    protected int sleepPeriod = 60;

    protected int wakeUpTime = 80;
    protected long startWakeUp;
    protected boolean starting;

    protected boolean stopping = false;
    protected int numClients = 0;
    protected String serverName;
    protected String uid;

    protected Logger log;

    protected GlobalReplicationManager grm = null;

    protected boolean isServerAlive = false;

    public ReplicationManager(int serverID, String name, int checkPeriod, GlobalReplicationManager grm )
    {
        log = Logger.getLogger(this.getClass().getName() + ":" + name);
        log.info("built for " + name);
//        System.out.println("*******************test");
        this.serverWatchedId = serverID;
        this.setCheckPeriod(checkPeriod);
        this.setServerName(name);
        this.grm = grm;

    }

    public boolean bindServer(boolean restart)
    {
        log.debug("binding to state rep bean.");
//        StateReplicator replicator = null;
        String homeName = "ejb/powerquotes/StateReplicator_" + this.serverWatchedId;


        try {
            InitialContext initialContext = new InitialContext();
            Object objRef = initialContext.lookup(homeName);

            StateReplicatorHome replicatorHome =
                (StateReplicatorHome) PortableRemoteObject.narrow(objRef, StateReplicatorHome .class);
            stateReplicator = replicatorHome.create();

            return true;
        }
        catch (javax.naming.CommunicationException ce)
        {
            // NamingException if naming exception encountered with the Context
            log.error("Caught a communication exception! " + ce.toString(), ce);
	        log.error("there's no JNDI server! -- I don't handle this!");
	        if (restart)
	        {
                this.haltServer();
                this.startServer();
	        }

        }
        catch(NamingException ne)
        {
            // NamingException if naming exception encountered with the Context
            log.error("Caught a naming exception! " + ne.toString(), ne);
	        log.error("I will halt and restart the server");
	        if (restart)
	        {
                this.haltServer();
                this.startServer();
	        }

//            throw new StockServerException();
        }
        catch(ClassCastException cce)
        {
            // ClassCastException if PortableRemoteObject cannot cast to UPSessionHome interface
            log.error("Caught a class cast exception! (bind)" + cce.toString());
//            throw new StockServerException();
        }
        catch(CreateException ce)
        {
            // CreateException if exception encountered whil creating the bean
            log.error("Caught a create exception! (bind)" + ce.toString());
//            throw new StockServerException();
        }
        catch(RemoteException re)
        {
            // RemoteException if exception encountered while connecting to server
            log.error("Caught a remote exception! (bind) " + re.toString());
//            throw new StockServerException();
	        log.error("I will halt and restart the server");
	        if (restart)
	        {
                this.haltServer();
                this.startServer();
	        }
        }

        stateReplicator = null;
        return false;


    }

    /**
     * Watches the server.
     * If the ping results in an exception the server is shutdown and restarted
     *
     * Should failed binding also generate a restart?
     */
    public void run()
    {
        log.debug("launched");
        while (!stopping)
        {
            if (stateReplicator == null)
            {
                bindServer(true);
            }
            else
            {
                try {
                    log.debug("pinging server");
                    numClients = stateReplicator.ping();
                    log.debug("server has " + numClients + " clients attached.");
                }
                catch (Exception e)
                {
                    //Server is dead!!!!
                    //Do something about it
                    isServerAlive = false;
                    log.error("my server (" + this.serverWatchedId + " / " + this.serverName + ") is dead!");
                    haltServer();
                    stateReplicator = null;
                    startServer();
                }

            }

            try {
//	            wait(1000 * this.getCheckPeriod());
                Thread.sleep(1000 * this.getCheckPeriod());
            }
            catch (InterruptedException ie)
            {
                //swallow the interruption
            }

        }

    }


    public void stopManager()
    {
        stopping = true;
    }

    public void halt()
    {
        //halt the server
        haltServer();
        this.stopManager();
    }

	protected static String jboss = "/afs/ece/class/ece841/public_html/team1/jboss-3.0.4_tomcat-4.1.12";
    protected void startServer()
    {
	    Process process = null;
        try {
            String cmd = "ssh " + uid + "@" + serverName + " " + jboss + "/bin/run.sh > /tmp/output.log";

	        log.info("starting server -- " + cmd);

 			process = Runtime.getRuntime().exec(cmd);
	        log.debug("going to wait for server to wake up");
            this.sleep(1000 * this.wakeUpTime);
	        log.info(serverName + " should be awake by now");

	        //here I need to initialize the stockcache on my server
	        for (int i = 0; i < 5 && this.stateReplicator == null; i++)
	        {
					log.debug("trying rebind " + i);
					this.bindServer(false);    //If this fails we're really in trouble!
					log.debug("finish rebind " + i);

//		        try
//		        {
			        if (this.stateReplicator == null)
			            this.sleep(1000 * this.wakeUpTime / 5);
//		        }
//		        catch (InterruptedException ie)
//		        {
//			        log.warn(ie);
//		        }
	        }
            if (this.stateReplicator == null)
            {
	            log.warn("didn't rebind, I'm exiting startServer.  We'll restart next time");
            }
	        else
            {

                // old way - lets the servers get the cache themselves
                // this.stateReplicator.initializeCache();

                // first get the cache from another server
                StockCache cache = grm.getOtherStockCache( serverWatchedId );
	            log.debug("cache output for " + this.getServerName());
	            GlobalReplicationManager.outputCache(cache);
                this.stateReplicator.initializeCache( cache );
	            log.debug("initialized cache");
                isServerAlive = true;
            }


//	        this.startWakeUp = System.currentTimeMillis();
//	        this.starting = true;

	        //can't wait - it stays running!
//	        if (process.waitFor() != 0)

       }
        catch (InterruptedException ie)
        {
	        log.warn("startServer interrupted: " + ie.toString(),  ie);
        }

        catch (IOException ioe)
        {
            log.error("exec failed " + ioe.toString(),  ioe);
        }

    }

	public void setWakeUpTime(int t)
	{
		this.wakeUpTime = t;
	}

    protected void haltServer()
    {
//	    try
//	    {
        isServerAlive = false;
		    boolean halted = CrashFaultInjector.killServer(serverName, uid);
//	    }

    }

    public int getCheckPeriod()
    {
        return this.sleepPeriod;
    }

    public void setCheckPeriod(int secs)
     {
         this.sleepPeriod = secs;
     }

    public int getNumClients()
    {
        return this.numClients;
    }

	public void setUID(String id)
	{
		this.uid = id;
	}

	public void setServerName(String n)
	{
		serverName = n;
	}

	public String getServerName()
	{
		return this.serverName;
	}

	public int getServerId()
	{
            return this.serverWatchedId;
	}

    public boolean isServerAlive()
    {
        return isServerAlive;
    }


    public StateReplicator getReplicator() {
        return stateReplicator;
    }
}











