package powerquotes.tests;

//--------------------------------------------------------------------------------------
/**
 * <p>Title: StockCacheTest </p>
 * <b> Category: Baseline Testing </b>
 * <b> Behavior: </b> This test interacts directly with the StockCache to determine
 * if it is behaving correctly.
 * <b> Inherits from: TestCase </b>  N/A
 * <b> Clients: </b> N/A
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//--------------------------------------------------------------------------------------

import junit.framework.TestCase;
import powerquotes.common.StockCache;
import powerquotes.common.StockInfo;
import powerquotes.common.ServerNotReadyException;
import powerquotes.ejb.*;
import powerquotes.datafeed.StockTicker;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import java.util.Collection;
import java.util.Iterator;
import java.rmi.RemoteException;
import java.rmi.Remote;
import javax.ejb.CreateException;

import org.apache.log4j.Logger;


public class StockCacheTest extends TestCase
{
    protected StockCache cache;
    protected StateReplicator stateBean;

    protected Logger log;


    protected static String suffix;

    protected void setUp()
    {
        org.apache.log4j.xml.DOMConfigurator.configure("log4j.xml");
        try
        {
            // connect to the home interface & create a State Replicator Session Bean
            Context initial = new InitialContext();
            String homeName = "ejb/powerquotes/StateReplicator" + suffix;
            Object objref = initial.lookup(homeName);
            StateReplicatorHome home = (StateReplicatorHome ) PortableRemoteObject.narrow(objref, StateReplicatorHome.class);
            stateBean = home.create();
        }
        catch( NamingException ne )
        {
            log.error( "Could not get the Home interface for TickReceiver. The server may not be up: " + ne );
            log.error( "Tying again in 5 seconds..." );
        }
        catch( CreateException ce ) {
            log.fatal( "Could not create TickReceieverSessionBean: " + ce );
            log.fatal( "Exiting..." );
            System.exit(2);
        } catch( RemoteException re ) {
            log.fatal( "Could not create TickReceieverSessionBean: " + re );
            log.fatal( "Exiting..." );
            System.exit(2);
        }

    }

    public void cacheInputTest()
    {
        String[] servers = {"ejb/powerquotes/TickReceiver" + suffix };
        StockTicker sim = new StockTicker("stockinfo.props", servers);
        sim.testSend(1, 10, 1);
    }

    public void cacheOutputTest()
    {
        try
        {
            System.out.println("have stateBean: " + stateBean);
            cache = stateBean.getStockCache();
            log.debug("got cache: " + cache);

            Collection stocks = cache.getAllStocks();
            for (Iterator i = stocks.iterator(); i.hasNext(); )
            {
                StockInfo si = (StockInfo)i.next();
                log.info(si.toString());
            }
        }
        catch(ServerNotReadyException se)
        {
            log.error("Caught a server not ready exception during cacheOutputTest!" + se.toString(), se);
        }        
        catch (RemoteException re)
        {
            log.error(re.toString(), re);
        }
    }

    public static void main(String args[] )
    {
        
        suffix = "_" + args[0];
        StockCacheTest test = new StockCacheTest();
        test.setUp();
        //test.cacheInputTest();
        test.cacheOutputTest();
    }


//    protected void output(String value)
//    {
//        System.out.println(value);
//    }
}

